-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local M = {}

local vehicleNames = 
{
	"scenario_player0",
	"aiCar",
}

local helper = require('scenario/scenariohelper')
local running = false
local playerWon = false

local function reset()
    running = false
    playerWon = false
end

local function fail(reason)
    reset()
    scenarios.finish({failed = reason})
end

local function success()
  local scenario = scenarios.getScenario()
  if not scenario then return end

  local finalTime = scenario.timer
  local minutes = math.floor(finalTime / 60);
  local seconds = finalTime - (minutes * 60);
  local timeStr = ''
  if minutes > 0 then
      timeStr = string.format("%02.0f:%05.2f", minutes, seconds)
  else
      timeStr = string.format("%0.2f", seconds) .. 's'
  end
  local result = {msg = "scenarios.jungle_rock_island.drag_race.win.msg", timeStr}
  scenarios.finish(result)
  reset()
end



local function onRaceInit()
    helper.queueLuaCommandByName('aiCar', 'extensions.loadModule("dragAi")')
    local aiTargetPos = helper.getVehicleByName('wp_ai_target'):getPosition()
    helper.queueLuaCommandByName('aiCar', 'dragAi.setTarget('..dumps({x= aiTargetPos.x, y = aiTargetPos.y, z = aiTargetPos.z})..')')
    helper.queueLuaCommandByName('aiCar', 'dragAi.getReady()')
    helper.queueLuaCommandByName("scenario_player0", "controller.mainController.setGearboxMode('realistic')")
	helper.queueLuaCommandByName("aiCar", "controller.mainController.setGearboxMode('arcade')")
end

local function onRaceStart()
    reset()

    helper.queueLuaCommandByName('aiCar', 'dragAi.go()')
    helper.trackVehicle('scenario_player0', 'playerCar')
    helper.trackVehicle('aiCar', 'aiCar')

    running = true
end

local function onRaceTick(raceTick)
  local scenario = scenarios.getScenario()
  if not scenario then return end    
  if running and scenario.timer >= 18 then
      fail("scenarios.jungle_rock_island.drag_race.fail.msg")
  end
end

local function onRaceWaypointReached(data)
  local playerVehicleId = be:getPlayerVehicleID(0)
  if data.vehicleId == playerVehicleId then
    local playerDistance = helper.getDistanceBetweenSceneObjects('scenario_player0', 'wp_target_distance')
    local aiDistance = helper.getDistanceBetweenSceneObjects('aiCar', 'wp_target_distance')
    playerWon = playerDistance <= aiDistance
  end
end

local function onRaceResult(time)
    if playerWon then
        success()
    else
        fail("scenarios.jungle_rock_island.drag_race.fail.msg")
    end
    helper.queueLuaCommandByName('aiCar', 'dragAi.reset()')
end

local function onScenarioRestarted(scenario)
    helper.queueLuaCommandByName('aiCar', 'dragAi.reset()')
end

--local defaultCarTypes = { "Car" }
local limitToNameContaining = {"Race", "Drag"}


function printObj(obj, hierarchyLevel, previousK) 
  if (hierarchyLevel == nil) then
    hierarchyLevel = 0
  elseif (hierarchyLevel == 4) then
    return 0
  end

  local whitespace = ""
  for i=0,hierarchyLevel,1 do
    whitespace = whitespace .. "-"
  end
  --io.write(whitespace)

  print "obj"
  print(obj)
  if (type(obj) == "table") then
    for k,v in pairs(obj) do
      --io.write(whitespace .. "-")
      if (type(v) == "table") then
		print(previousK.."="..tostring(v).." [table]")
		previousK = previousK.."/"..k
        printObj(v, hierarchyLevel+1, previousK)
      else
		--print "k"
		--print(k)
		--print "v"
        print(previousK.."/"..k.."="..tostring(v))
      end           
    end
  else
	print "obj"
    print(obj)
  end
end

local function has_value (tab, val)
    for index, value in ipairs(tab) do
        if value == val then
            return true
        end
    end

    return false
end

local function onPreStart(scenario)
scenario.demoNames = {}
	-- Get the list of vehicles
	-- local vehicleList = vehicles.getVehicleList().list
	
	-- Get a new random seed
	math.randomseed(os.time())
	--validCarTypes = validCarTypes or defaultCarTypes
	
	local minimumPC = 4
	local performanceClassesFound = {}
	local performanceClassesFoundString = ""
	local ValidCarsList = {}
	local counter = 0
	local configList = vehicles.getConfigList().configs
	local valid = false

	local thisRunCarNameLimited = limitToNameContaining[tableChooseRandomKey(limitToNameContaining)]
	-- Ensures that both cars are of the same vehicle type (car or truck)
	--local thisRunCarTypeLimited = validCarTypes[tableChooseRandomKey(validCarTypes)]
	
	print("Limited to cars containing the word "..thisRunCarNameLimited.." in it's name and with a performace class of minimum "..minimumPC )
	
	for _, config in pairs(configList) do
		--for _, validType in pairs(validCarTypes) do
				--if config.aggregates.Type[validType] then
					--for _, limitedName in pairs(limitToNameContaining) do
						if string.match(config.Name, thisRunCarNameLimited) then
							if (tonumber(config["Performance Class"]) >= tonumber(minimumPC)) then 
							-- printObj(config,0,"/")
								if has_value(performanceClassesFound,config["Performance Class"]) == false then
									table.insert(performanceClassesFound,config["Performance Class"])
									performanceClassesFoundString = performanceClassesFoundString..","..config["Performance Class"]
								end
								ValidCarsList[counter] = config
								counter = counter + 1
							end
						end
					--end
					
				--end
		--end
	end
	
	local restrictedToPerformanceClass = performanceClassesFound[tableChooseRandomKey(performanceClassesFound)]
	
	print("Performanceclasses to choose from: "..tostring(performanceClassesFoundString))
	print("Restricting this event to cars with performance class: "..restrictedToPerformanceClass)
	
	local ValidCarsListWashed = {}
	for _, VCL in pairs(ValidCarsList) do
		print(VCL.Name.."="..VCL["Performance Class"])
		if (tonumber(VCL["Performance Class"]) == tonumber(restrictedToPerformanceClass)) then
			table.insert(ValidCarsListWashed,VCL)
		end
	end
	
	print("Potential contenders:")
	for _, VCL in pairs(ValidCarsListWashed) do
		print(VCL.Name.."="..VCL["Performance Class"])
	end
	
	for i, carName in pairs(vehicleNames) do
		local config = ValidCarsListWashed[tableChooseRandomKey(ValidCarsListWashed)]
		
		local selectedVehicle = config.Name or 'Unknown'
					
		local jbeam = config.model_key
		local partConfig = string.format('vehicles/%s/%s.pc', jbeam, config.key)
		local color = config.default_color or 'TransparentWhite'
		
		TorqueScript.eval([[
			]]..carName..[[.JBeam = "]] .. jbeam .. [[";
			]]..carName..[[.partConfig = "]] .. partConfig .. [[";
			]]..carName..[[.color = "]] .. color .. [[";
			]]..carName..[[.requestReload();
		]])
		scenario.demoNames[i - 1] = config.key
	end	
		
end

local function onScenarioChange(scenario)
	if scenario and scenario.state == 'pre-start' then
		onPreStart(scenario)
	end
end
-- Finished adding stuff
M.onRaceStart = onRaceStart
M.onRaceInit = onRaceInit
M.onRaceTick = onRaceTick
M.onRaceResult = onRaceResult
M.onRaceWaypointReached = onRaceWaypointReached
M.onScenarioRestarted = onScenarioRestarted

-- added by dagalufh, original code by Scepheo
M.onScenarioChange = onScenarioChange

return M
